@@ -1573,112 +1573,42 @@ oo_load_mapping(perl_yaml_xs_t *self)
     return hash_ref;
 }
 
-SV *
-oo_load_scalar(perl_yaml_xs_t *self)
+int
+_match_core_schema(char *string)
 {
-    SV *scalar;
-    char *string = (char *)self->event.data.scalar.value;
-    yaml_scalar_style_t style = self->event.data.scalar.style;
-    char *anchor = (char *)self->event.data.scalar.anchor;
-    char *tag = (char *)self->event.data.scalar.tag;
-    STRLEN length = (STRLEN)self->event.data.scalar.length;
-    I32 flags = 0;
     int i = 0;
-    if (tag) {
-        if (strEQ(tag, YAML_STR_TAG)) {
-            style = YAML_SINGLE_QUOTED_SCALAR_STYLE;
-        }
-    }
+    int got_decimal = 0;
+    int got_mantissa = 0;
+    int is_float = 0;
 
-    if (style != YAML_PLAIN_SCALAR_STYLE) {
-        goto return_string;
-    }
-
-    /* bool true */
     if (strEQ(string, "true") || strEQ(string, "TRUE") || strEQ(string, "True")) {
-#ifdef PERL_HAVE_BOOLEANS
-        scalar = newSVsv(&PL_sv_yes);
-#else
-        scalar = &PL_sv_yes;
-#endif
-        if (tag && ! strEQ(tag, YAML_BOOL_TAG)) {
-            croak("%s", oo_loader_error_msg( self, form("Invalid tag '%s' for value '%s'", tag, string)));
-        }
-        goto return_scalar;
+        return YAML_XS_SCALAR_TYPE_BOOL_TRUE;
     }
-
-    /* bool false */
     if (strEQ(string, "false") || strEQ(string, "FALSE") || strEQ(string, "False")) {
-#ifdef PERL_HAVE_BOOLEANS
-        scalar = newSVsv(&PL_sv_no);
-#else
-        scalar = &PL_sv_no;
-#endif
-        if (tag && ! strEQ(tag, YAML_BOOL_TAG)) {
-            croak("%s", oo_loader_error_msg( self, form("Invalid tag '%s' for value '%s'", tag, string)));
-        }
-        goto return_scalar;
+        return YAML_XS_SCALAR_TYPE_BOOL_FALSE;
     }
-
-    /* null */
     if (strEQ(string, "null") || strEQ(string, "NULL") || strEQ(string, "Null") || strEQ(string, "~") || strEQ(string, "")) {
-        scalar = newSV(0);
-        if (tag && ! strEQ(tag, YAML_NULL_TAG)) {
-            croak("%s", oo_loader_error_msg( self, form("Invalid tag '%s' for value '%s'", tag, string)));
-        }
-        goto return_scalar;
+        return YAML_XS_SCALAR_TYPE_NULL;
     }
-
-    /* inf */
     if (
         strEQ(string, ".INF") || strEQ(string, ".Inf") || strEQ(string, ".inf")
         || strEQ(string, "+.INF") || strEQ(string, "+.Inf") || strEQ(string, "+.inf")
         || strEQ(string, "-.INF") || strEQ(string, "-.Inf") || strEQ(string, "-.inf")
         ) {
-        if (tag && ! strEQ(tag, YAML_FLOAT_TAG)) {
-            croak("%s", oo_loader_error_msg( self, form("Invalid tag '%s' for value '%s'", tag, string)));
-        }
-        if (string[0] == 45) {
-            scalar = newSVnv(-NV_INF);
-        }
-        else {
-            scalar = newSVnv(NV_INF);
-        }
-        goto return_scalar;
+        return YAML_XS_SCALAR_TYPE_FLOAT_INF;
     }
-
-    /* nan */
-    if (
-        strEQ(string, ".NAN") || strEQ(string, ".NaN") || strEQ(string, ".nan")
-        ) {
-        NV nan = NV_NAN;
-        string++;
-        length--;
-        if (tag && ! strEQ(tag, YAML_FLOAT_TAG)) {
-            croak("%s", oo_loader_error_msg( self, form("Invalid tag '%s' for value '%s'", tag, string)));
-        }
-        scalar = newSVnv(nan);
-        goto return_scalar;
+    if (strEQ(string, ".NAN") || strEQ(string, ".NaN") || strEQ(string, ".nan")) {
+        return YAML_XS_SCALAR_TYPE_FLOAT_NAN;
     }
-
-    /* oct */
     if (string[0] == 48 && string[1] == 111) {
         for (i=2; i < strlen(string); i++) {
             if (!( string[i] >= 48 && string[i] <= 55)) {       // 0-7
-                goto return_string;
-                break;
+                return YAML_XS_SCALAR_TYPE_STRING;
             }
         }
-        string += 2;
-        length -= 2;
-        int num = grok_oct(string, &length, &flags, NULL);
-        scalar = newSViv((int) num);
-        goto return_scalar;
+        return YAML_XS_SCALAR_TYPE_INT_OCT;
     }
-
-    /* hex */
     if (string[0] == 48 && string[1] == 120) {
-        //fprintf(stderr, "string: %s\n", string);
         for (i=2; i < strlen(string); i++) {
             if (!(
                    (string[i] >= 48 && string[i] <= 57)    // 0-10
@@ -1686,25 +1616,15 @@ oo_load_scalar(perl_yaml_xs_t *self)
                 || (string[i] >= 65 && string[i] <= 70)    // A-F
                 )
             ) {
-                goto return_string;
-                break;
+                return YAML_XS_SCALAR_TYPE_STRING;
             }
         }
-        string += 2;
-        length -= 2;
-        int num = grok_hex(string, &length, &flags, NULL);
-        scalar = newSViv((int) num);
-        goto return_scalar;
+        return YAML_XS_SCALAR_TYPE_INT_HEX;
     }
 
-    /* float or int */
-    i = 0;
     if (string[0] == 43 || string[0] == 45) { // +-
         i++;
     }
-    int got_decimal = 0;
-    int got_mantissa = 0;
-    int is_float = 0;
     while (i < strlen(string)) {
         if (string[i] >= 48 && string[i] <= 57) { // 0-9
             got_decimal = 1;
@@ -1727,7 +1647,7 @@ oo_load_scalar(perl_yaml_xs_t *self)
         i++;
     }
     if (! got_mantissa && ! got_decimal) {
-        goto return_string;
+        return YAML_XS_SCALAR_TYPE_STRING;
     }
     int got_exponent = 0;
     if (i < strlen(string) && (string[i] == 101 || string[i] == 69)) { // eE
@@ -1745,14 +1665,135 @@ oo_load_scalar(perl_yaml_xs_t *self)
             i++;
         }
         if (! got_exponent) {
-            goto return_string;
+            return YAML_XS_SCALAR_TYPE_STRING;
         }
     }
     if (i < strlen(string)) {
+        return YAML_XS_SCALAR_TYPE_STRING;
+    }
+    if (is_float) {
+        return YAML_XS_SCALAR_TYPE_FLOAT;
+    }
+    else {
+        return YAML_XS_SCALAR_TYPE_INT;
+    }
+    return YAML_XS_SCALAR_TYPE_STRING;
+}
+
+SV *
+oo_load_scalar(perl_yaml_xs_t *self)
+{
+    SV *scalar;
+    char *string = (char *)self->event.data.scalar.value;
+    yaml_scalar_style_t style = self->event.data.scalar.style;
+    char *anchor = (char *)self->event.data.scalar.anchor;
+    char *tag = (char *)self->event.data.scalar.tag;
+    STRLEN length = (STRLEN)self->event.data.scalar.length;
+    I32 flags = 0;
+    int scalar_type = 0;
+    if (tag) {
+        if (strEQ(tag, YAML_STR_TAG)) {
+            style = YAML_SINGLE_QUOTED_SCALAR_STYLE;
+        }
+    }
+
+    if (style != YAML_PLAIN_SCALAR_STYLE) {
+        goto return_string;
+    }
+
+    scalar_type = _match_core_schema(string);
+
+    /* bool true */
+    if (scalar_type == YAML_XS_SCALAR_TYPE_BOOL_TRUE) {
+#ifdef PERL_HAVE_BOOLEANS
+        scalar = newSVsv(&PL_sv_yes);
+#else
+        scalar = &PL_sv_yes;
+#endif
+        if (tag && ! strEQ(tag, YAML_BOOL_TAG)) {
+            croak("%s", oo_loader_error_msg( self, form("Invalid tag '%s' for value '%s'", tag, string)));
+        }
+        goto return_scalar;
+    }
+
+    /* bool false */
+    if (scalar_type == YAML_XS_SCALAR_TYPE_BOOL_FALSE) {
+#ifdef PERL_HAVE_BOOLEANS
+        scalar = newSVsv(&PL_sv_no);
+#else
+        scalar = &PL_sv_no;
+#endif
+        if (tag && ! strEQ(tag, YAML_BOOL_TAG)) {
+            croak("%s", oo_loader_error_msg( self, form("Invalid tag '%s' for value '%s'", tag, string)));
+        }
+        goto return_scalar;
+    }
+
+    /* null */
+    if (scalar_type == YAML_XS_SCALAR_TYPE_NULL) {
+        scalar = newSV(0);
+        if (tag && ! strEQ(tag, YAML_NULL_TAG)) {
+            croak("%s", oo_loader_error_msg( self, form("Invalid tag '%s' for value '%s'", tag, string)));
+        }
+        goto return_scalar;
+    }
+
+    /* inf */
+    if (scalar_type == YAML_XS_SCALAR_TYPE_FLOAT_INF) {
+        if (tag && ! strEQ(tag, YAML_FLOAT_TAG)) {
+            croak("%s", oo_loader_error_msg( self, form("Invalid tag '%s' for value '%s'", tag, string)));
+        }
+        if (string[0] == 45) {
+            scalar = newSVnv(-NV_INF);
+        }
+        else {
+            scalar = newSVnv(NV_INF);
+        }
+        goto return_scalar;
+    }
+
+    /* nan */
+    if (scalar_type == YAML_XS_SCALAR_TYPE_FLOAT_NAN) {
+        NV nan = NV_NAN;
+        string++;
+        length--;
+        if (tag && ! strEQ(tag, YAML_FLOAT_TAG)) {
+            croak("%s", oo_loader_error_msg( self, form("Invalid tag '%s' for value '%s'", tag, string)));
+        }
+        scalar = newSVnv(nan);
+        goto return_scalar;
+    }
+
+    /* oct */
+    if (scalar_type == YAML_XS_SCALAR_TYPE_INT_OCT) {
+        if (tag && ! strEQ(tag, YAML_INT_TAG)) {
+            croak("%s", oo_loader_error_msg( self, form("Invalid tag '%s' for value '%s'", tag, string)));
+        }
+        string += 2;
+        length -= 2;
+        int num = grok_oct(string, &length, &flags, NULL);
+        scalar = newSViv((int) num);
+        goto return_scalar;
+    }
+
+    /* hex */
+    if (scalar_type == YAML_XS_SCALAR_TYPE_INT_HEX) {
+        if (tag && ! strEQ(tag, YAML_INT_TAG)) {
+            croak("%s", oo_loader_error_msg( self, form("Invalid tag '%s' for value '%s'", tag, string)));
+        }
+        string += 2;
+        length -= 2;
+        int num = grok_hex(string, &length, &flags, NULL);
+        scalar = newSViv((int) num);
+        goto return_scalar;
+    }
+
+    /* float or int */
+    if (scalar_type != YAML_XS_SCALAR_TYPE_INT && scalar_type != YAML_XS_SCALAR_TYPE_FLOAT) {
         goto return_string;
     }
     scalar = newSVpvn(string, length);
-    if (is_float) {
+    if (scalar_type == YAML_XS_SCALAR_TYPE_FLOAT) {
         if (tag && ! strEQ(tag, YAML_FLOAT_TAG)) {
             croak("%s", oo_loader_error_msg( self, form("Invalid tag '%s' for value '%s'", tag, string)));
         }
@@ -2025,39 +2066,9 @@ oo_dump_scalar(perl_yaml_xs_t *self, SV *node)
     else {
         node_clone = sv_mortalcopy(node);
         string = SvPV_nomg(node_clone, string_len);
-        if (
-            strEQ(string, "true") || strEQ(string, "TRUE") || strEQ(string, "True")
-            || strEQ(string, "false") || strEQ(string, "FALSE") || strEQ(string, "False")
-            || strEQ(string, "null") || strEQ(string, "NULL") || strEQ(string, "Null") || strEQ(string, "~") || strEQ(string, "")
-            || strEQ(string, ".INF") || strEQ(string, ".Inf") || strEQ(string, ".inf")
-            || strEQ(string, "+.INF") || strEQ(string, "+.Inf") || strEQ(string, "+.inf")
-            || strEQ(string, "-.INF") || strEQ(string, "-.Inf") || strEQ(string, "-.inf")
-            || strEQ(string, ".NAN") || strEQ(string, ".NaN") || strEQ(string, ".nan")
-        ) {
+        if (_match_core_schema(string)) {
             style = YAML_SINGLE_QUOTED_SCALAR_STYLE;
         }
-        else if (
-            string[0] == 43 || string[0] == 45 || string[0] == 46
-            || (string[0] >= 48 && string[0] <= 57)) {
-            dSP;
-            length = strlen(string);
-            SV *scalar = newSVpvn(string, length);
-            ENTER;
-            SAVETMPS;
-            PUSHMARK(sp);
-            XPUSHs(scalar);
-            PUTBACK;
-            is_num = call_pv("YAML::XS::__is_number", G_SCALAR);
-            SPAGAIN;
-            is_num = (POPi);
-
-            PUTBACK;
-            FREETMPS;
-            LEAVE;
-            if (is_num) {
-                style = YAML_SINGLE_QUOTED_SCALAR_STYLE;
-            }
-        }
     }
 
     if (! yaml_scalar_event_initialize(
